@@ -1,9 +1,8 @@
-
 /*
  * @(#)bitmap.c
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2004, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -65,8 +64,7 @@ typedef struct BITMAPFILEHEADER
   PILRC_USHORT bfReserved2;
   PILRC_USHORT bfOffBits1;
   PILRC_USHORT bfOffBits2;
-}
-BITMAPFILEHEADER;
+} BITMAPFILEHEADER;
 
 typedef struct BITMAPINFOHEADER
 {
@@ -81,8 +79,7 @@ typedef struct BITMAPINFOHEADER
   long biYPelsPerMeter;
   PILRC_ULONG biClrUsed;
   PILRC_ULONG biClrImportant;
-}
-BITMAPINFOHEADER;
+} BITMAPINFOHEADER;
 
 typedef struct RGBQUAD
 {
@@ -90,21 +87,19 @@ typedef struct RGBQUAD
   PILRC_BYTE rgbGreen;
   PILRC_BYTE rgbRed;
   PILRC_BYTE rgbReserved;
-}
-RGBQUAD;
+} RGBQUAD;
 
 typedef struct BITMAPINFO
 {
   BITMAPINFOHEADER bmiHeader;
-  RGBQUAD bmiColors[1];
-}
-BITMAPINFO;
+  RGBQUAD bmiColors[1]; /* size is indefinite */
+} BITMAPINFO;
 #endif
 
-#define BI_RGB					0
-#define BI_RLE4					1
-#define BI_RLE8					2
-#define BI_BITFIELDS		3
+#define BI_RGB         0
+#define BI_RLE4        1
+#define BI_RLE8        2
+#define BI_BITFIELDS   3
                                                                 // *INDENT-OFF*
 /*
  * The 1bit-2 color system palette for Palm Computing Devices.
@@ -267,7 +262,7 @@ static void BMP_ConvertX11Bitmap(RCBITMAP *, PILRC_BYTE *, int);
 static void BMP_ConvertPNMBitmap(RCBITMAP *, PILRC_BYTE *, int, int, BOOL, int *);
 static void BMP_CompressBitmap(RCBITMAP *, int, int, BOOL, BOOL);
 static void BMP_CompressDumpBitmap(RCBITMAP *, int, int, BOOL, BOOL, BOOL, BOOL, int, int *);
-static void BMP_InvalidExtension(char *);
+static void BMP_InvalidExtension(const char *);
 
 static void BMP_FillBitmapV3Header(RCBITMAP *, RCBITMAP_V3 *, int *, int);
                                                                 // *INDENT-ON*
@@ -780,9 +775,9 @@ BMP_SetBits24bpp(int cx,
 
   cbRow = BMP_CbRow(cx, 24, cBitsAlign);
   pb += cbRow * y + (x * 3);
-  *pb = (PILRC_BYTE) ((bits & 0xFF0000) >> 16);  // red
-  *pb++ = (PILRC_BYTE) ((bits & 0x00FF00) >> 8); // green
-  *pb++ = (PILRC_BYTE) (bits & 0xFF);            // blue
+  pb[0] = (PILRC_BYTE) ((bits & 0xFF0000) >> 16); // red
+  pb[1] = (PILRC_BYTE) ((bits & 0x00FF00) >> 8);  // green
+  pb[2] = (PILRC_BYTE) (bits & 0xFF);             // blue
 }
 
 /**
@@ -807,10 +802,10 @@ BMP_SetBits32bpp(int cx,
 
   cbRow = BMP_CbRow(cx, 32, cBitsAlign);
   pb += cbRow * y + (x * 4);
-  *pb = (PILRC_BYTE) ((bits & 0xFF000000) >> 24);       // alpha
-  *pb++ = (PILRC_BYTE) ((bits & 0x00FF0000) >> 16);     // red
-  *pb++ = (PILRC_BYTE) ((bits & 0x0000FF00) >> 8);      // green
-  *pb++ = (PILRC_BYTE) (bits & 0xFF);            // blue
+  pb[0] = (PILRC_BYTE) ((bits & 0xFF000000) >> 24); // alpha
+  pb[1] = (PILRC_BYTE) ((bits & 0x00FF0000) >> 16); // red
+  pb[2] = (PILRC_BYTE) ((bits & 0x0000FF00) >> 8);  // green
+  pb[3] = (PILRC_BYTE) (bits & 0xFF);               // blue
 }
 
 /**
@@ -1758,13 +1753,8 @@ WriteGreyTbmp(RCBITMAP * rcbmp,
   free(index);
 
   if (ningreys > (int)(outmaxval + 1))
-  {
-    char buffer[120];
-
-    sprintf(buffer, "%d input grey levels converted to only %ld",
-            ningreys, outmaxval + 1);
-    WarningLine(buffer);
-  }
+    WarningLine("%d input grey levels converted to only %ld",
+                ningreys, outmaxval + 1);
 }
 
 static int
@@ -1778,8 +1768,7 @@ WriteIndexedColorTbmp(RCBITMAP * rcbmp,
   {
     struct rgb key;
     unsigned char index;
-  }
-  table[N];
+  } table[N];
 
   unsigned char *outp = rcbmp->pbBits;
   int x, y, h, nentries, ninputcolors, noutputcolors;
@@ -1873,10 +1862,7 @@ WriteIndexedColorTbmp(RCBITMAP * rcbmp,
 
   if (ninputcolors > 256)
   {
-    char buffer[120];
-
-    sprintf(buffer, "%d input colors converted to only 256", ninputcolors);
-    WarningLine(buffer);
+    WarningLine("%d input colors converted to only 256", ninputcolors);
     ninputcolors = 256;
   }
 
@@ -2539,26 +2525,18 @@ BMP_CompressDumpBitmap(RCBITMAP * rcbmp,
           ((rcbmp->cx != stdIconSize_x) || (rcbmp->cy != stdIconSize_x)) &&
           ((rcbmp->cx != stdIconSize_x) || (rcbmp->cy != stdIconSize_y)) &&
           ((rcbmp->cx != stdIconSize_y) || (rcbmp->cy != stdIconSize_y)))
-      {
-		  char buffer[128];
-		  sprintf(buffer, "Icon resource not %dx%d, %dx%d or %dx%d (preferred)",
-		  	stdIconSize_x, stdIconSize_x, 
-		  	stdIconSize_x, stdIconSize_y,
-		  	stdIconSize_y, stdIconSize_y);
-		  WarningLine(buffer);
-      }
+        WarningLine("Icon resource not %dx%d, %dx%d or %dx%d (preferred)",
+                    stdIconSize_x, stdIconSize_x,
+                    stdIconSize_x, stdIconSize_y,
+                    stdIconSize_y, stdIconSize_y);
       break;
 
     case 1001:
       if (!vfAllowBadIconSizes &&
           (rcbmp->cx != stdSmallIconSize_x) &&
           (rcbmp->cy != stdSmallIconSize_y))
-      {
-		  char buffer[128];
-		  sprintf(buffer, "Small icon resource not %dx%d",
-		  	stdSmallIconSize_x, stdSmallIconSize_y);
-          WarningLine(buffer);
-      }
+        WarningLine("Small icon resource not %dx%d",
+                    stdSmallIconSize_x, stdSmallIconSize_y);
       break;
 
     default:
@@ -2708,15 +2686,11 @@ BMP_CompressDumpBitmap(RCBITMAP * rcbmp,
  * @param fileName the source file name
  */
 static void
-BMP_InvalidExtension(char *fileName)
+BMP_InvalidExtension(const char *fileName)
 {
-  char pchLine[300];
-
-  sprintf(pchLine,
-          "Bitmap file extension not recognized for file %s\n"
-          "\tSupported extensions: .BMP, .pbitm, .xbm and .pbm/.ppm/.pnm",
-          fileName);
-  ErrorLine(pchLine);
+  ErrorLine("Bitmap file extension not recognized for file %s"
+            " (Supported extensions: .BMP, .pbitm, .xbm and .pbm/.ppm/.pnm)",
+            fileName);
 }
 
 /**
@@ -2732,8 +2706,7 @@ BMP_InvalidExtension(char *fileName)
  * @param bootscreen	should this bitmap be prepared for size & crc header add on ?
  * @param density    does this bitmap is a new version
  */
-extern void
-DumpBitmap(char *fileName,
+void DumpBitmap(const char *fileName,
            int isIcon,
            int compress,
            int bitmaptype,
@@ -2744,7 +2717,8 @@ DumpBitmap(char *fileName,
            int density)
 {
   PILRC_BYTE *pBMPData;
-  char *pchExt;
+  const char *pchExt;
+  char *pathName;
   FILE *pFile;
   long size;
   RCBITMAP rcbmp;
@@ -2810,14 +2784,9 @@ DumpBitmap(char *fileName,
                  (bitmaptype == rwBitmapColor32k));
 
   // determine the size of the resource to load
-  fileName = FindAndOpenFile(fileName, "rb", &pFile);
+  pathName = FindAndOpenFile(fileName, "rb", &pFile);
   if (pFile == NULL)
-  {
-    char pchLine[200];
-
-    sprintf(pchLine, "Could not find Resource %s.", fileName);
-    ErrorLine(pchLine);
-  }
+    ErrorLine("Could not find Resource %s.", fileName);
 
   fseek(pFile, 0, SEEK_END);
   size = ftell(pFile);
@@ -2826,20 +2795,15 @@ DumpBitmap(char *fileName,
   // alocate memory and load the resource in
   pBMPData = malloc(size);
   if (pBMPData == NULL)
-  {
-    char pchLine[200];
-
-    sprintf(pchLine, "Resource %s too big to fit in memory", fileName);
-    ErrorLine(pchLine);
-  }
+    ErrorLine("Resource %s too big to fit in memory", pathName);
   fread(pBMPData, 1, size, pFile);
   fclose(pFile);
 
   // obtain the file extension
-  pchExt = strrchr(fileName, '.');
-  if ((strlen(fileName) < 5) || (pchExt == NULL))
+  pchExt = strrchr(pathName, '.');
+  if ((strlen(pathName) < 5) || (pchExt == NULL))
   {
-    BMP_InvalidExtension(fileName);
+    BMP_InvalidExtension(pathName);
   }
   pchExt++;
 
@@ -2878,9 +2842,10 @@ DumpBitmap(char *fileName,
   }
   else
   {
-    BMP_InvalidExtension(fileName);
+    BMP_InvalidExtension(pathName);
   }
 
   // clean up
   free(pBMPData);
+  free(pathName);
 }
