@@ -2,7 +2,7 @@
  * @(#)font.c
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2004, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -31,10 +31,10 @@
 // #include <memory.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include "pilrc.h"
-#include "util.h"
 
 /*
  * Header definitions and token types 
@@ -62,10 +62,13 @@
 #define g_width 22                               /* width token */
 #define g_bitmap 99                              /* bitmap data */
 
+
+#define kArraySize 258
+
 /*
  * some foreign font handing function 
  */
-int (*pfnChkCode) (unsigned char *cp,
+int (*pfnChkCode) (const unsigned char *cp,
                    int *pdx);
 
 /*
@@ -84,13 +87,13 @@ typedef struct FontCharInfoType
 {
   char offset;
   char width;
-}
-FontCharInfoType;
+} FontCharInfoType;
+
 FontCharInfoType *fntOW[256];
 unsigned int fntH[256];
 
 static int
-IsDBCSNone(unsigned char *cp,
+IsDBCSNone(const unsigned char *cp,
            int *pdxChar)
 {
   return 0;
@@ -102,7 +105,7 @@ IsDBCSNone(unsigned char *cp,
 |		Check the double byte char is BIG5 coded
 -------------------------------------------------------------DLIN------------*/
 static int
-IsBIG5(unsigned char *cp,
+IsBIG5(const unsigned char *cp,
        int *pdxChar)
 {
 
@@ -142,7 +145,7 @@ IsBIG5(unsigned char *cp,
 |		Check the double byte char is Japanese coded
 -------------------------------------------------------------DLIN------------*/
 static int
-IsJapanese(unsigned char *cp,
+IsJapanese(const unsigned char *cp,
            int *pdxChar)
 {
 
@@ -151,11 +154,10 @@ IsJapanese(unsigned char *cp,
     *pdxChar = 5;
     return 1;
   }
-  else
-    if ((((*cp >= 0x81) && (*cp <= 0x9f)) ||
-         ((*cp >= 0xe0) && (*cp <= 0xef))) &&
-        (((*(cp + 1) >= 0x40) && (*(cp + 1) <= 0x7e)) ||
-         ((*(cp + 1) >= 0x80) && (*(cp + 1) <= 0xfc))))
+  else if ((((*cp >= 0x81) && (*cp <= 0x9f)) ||
+            ((*cp >= 0xe0) && (*cp <= 0xef))) &&
+           (((*(cp + 1) >= 0x40) && (*(cp + 1) <= 0x7e)) ||
+            ((*(cp + 1) >= 0x80) && (*(cp + 1) <= 0xfc))))
   {
     *pdxChar = 9;                                /* not sure about this */
     return 2;
@@ -170,7 +172,7 @@ IsJapanese(unsigned char *cp,
 |	Check the double byte char is Korean coded(Large Font, for HanMe)
 -----------------------------------------------------------------------------*/
 static int
-IsKoreanHanme(unsigned char *cp,
+IsKoreanHanme(const unsigned char *cp,
               int *pdxChar)
 {
 
@@ -195,7 +197,7 @@ IsKoreanHanme(unsigned char *cp,
 |	Check the double byte char is Korean coded(Small Font, for Hantiip)
 -----------------------------------------------------------------------------*/
 static int
-IsKoreanHantip(unsigned char *cp,
+IsKoreanHantip(const unsigned char *cp,
                int *pdxChar)
 {
 
@@ -213,17 +215,23 @@ IsKoreanHantip(unsigned char *cp,
   return 0;
 }
 
+static void
+DiagnosticX(BOOL error, const char *szFormat, ...)
+{
+  va_list args;
+  va_start(args, szFormat);
+  Diagnostic(error, filename, lineno, szFormat, &args);
+  va_end(args);
+}
+
 /*
  * Report an error with line number and filename 
  */
 
 static void
-ErrorX(char *s)
+ErrorX(const char *s)
 {
-  char er[512];
-
-  sprintf(er, "ERROR: %s, line %u, %s", s, lineno, filename);
-  Error(er);
+  DiagnosticX(fTrue, "%s", s);
 }
 
 /*
@@ -231,9 +239,9 @@ ErrorX(char *s)
  */
 
 static void
-WarningX(char *s)
+WarningX(const char *s)
 {
-  fprintf(stderr, "WARNING: %s, line %u, %s", s, lineno, filename);
+  DiagnosticX(fFalse, "%s", s);
 }
 
 /*
@@ -320,24 +328,24 @@ ParseLine(char *s,
 
 static void
 CloseGlyph(int *header,
-           int *width,
+           int width,
            int *row,
            int *col,
            int autoWidth,
            int autoRectWidth)
 {
-  if (*width > header[h_maxWidth])
+  if (width > header[h_maxWidth])
   {
     if (autoWidth)
-      header[h_maxWidth] = *width;
+      header[h_maxWidth] = width;
     else
       WarningX("Width exceeds maxWidth definition");
   }
 
-  if (*width > header[h_fRectWidth])
+  if (width > header[h_fRectWidth])
   {
     if (autoRectWidth)
-      header[h_fRectWidth] = *width;
+      header[h_fRectWidth] = width;
     else
       WarningX("Width exceeds fRectWidth definition");
   }
@@ -348,7 +356,7 @@ CloseGlyph(int *header,
     ErrorX("Invalid height on previous glyph");
 
   *row = 0;
-  *col += *width;
+  *col += width;
 }
 
 /*
@@ -372,15 +380,14 @@ CloseGlyph(int *header,
  */
 
 void
-DumpFont(char *pchFileName,
+DumpFont(const char *pchFileName,
          int fntNo)
 {
-#define		kArraySize		258
   FILE *in;
   char s[kArraySize], *s1, *bitmap[kArraySize];
   unsigned short int coltable[kArraySize];
   size_t x;
-  int token, value, header[13];
+  int token, value, header[14];
   int curChar = -1;
   int autoWidth = 1;
   int autoRectWidth = 1;
@@ -418,10 +425,10 @@ DumpFont(char *pchFileName,
         break;
     }
 
-    if (s1[0] && (s[0] != 47 || s[1] != 47))
+    if (s1[0] && (s[0] != '/' || s[1] != '/'))
     {                                            /* skip blank lines and comment lines */
       ParseLine(s1, &token, &value);
-      if (token < g_glyph)
+      if (token <= h_leading)
       {
         if (curChar >= 0)
           ErrorX("Header must precede glyphs");
@@ -442,7 +449,7 @@ DumpFont(char *pchFileName,
       if (token == g_glyph)
       {
         if (curChar >= 0)
-          CloseGlyph(header, &width, &row, &col, autoWidth, autoRectWidth);
+          CloseGlyph(header, width, &row, &col, autoWidth, autoRectWidth);
 
         if (value == -1)
         {
@@ -524,7 +531,7 @@ DumpFont(char *pchFileName,
 
   }
   if (!feof(in))
-    Error2("Error reading file: ", filename);
+    Error("Error reading file: %s", filename);
 
   fclose(in);
 
@@ -549,7 +556,7 @@ DumpFont(char *pchFileName,
    * Add the extra column info used to compute the last char width (even if it is the missing char) 
    */
   coltable[curChar + 1] = coltable[curChar] + fntOW[fntNo][curChar].width;
-  CloseGlyph(header, &width, &row, &col, autoWidth, autoRectWidth);
+  CloseGlyph(header, width, &row, &col, autoWidth, autoRectWidth);
   fntH[fntNo] = header[h_fRectHeight];
 
   header[h_rowWords] = (col + 15) / 16;
@@ -595,6 +602,8 @@ DumpFont(char *pchFileName,
   // dump CharInfoTag
   DumpBytes(&fntOW[fntNo][header[h_firstChar]],
             (header[h_lastChar] - header[h_firstChar] + 1 + missingChar) * 2);
+
+  free(filename);
 }
 
 
@@ -607,7 +616,6 @@ DumpFont(char *pchFileName,
 void
 DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * fontFamilyEntries)
 {
-#define kArraySize 258
   FILE *in;
   char s[kArraySize], *s1, *bitmap[kArraySize];
   unsigned short int coltable[kArraySize];
@@ -620,13 +628,10 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
   unsigned int aGlyphBitsOffset = 0;
 
   FontCharInfoType *singleOW = NULL;
-  unsigned int singleH;
-
-  int badGlyphs[kArraySize];
-  int *badGlyph = &badGlyphs[0];
+  unsigned int singleH = 0;
 
   if (densityCount < 2)
-    Error("FontFamily must have 2 fonts with density 72 and 144.");
+    ErrorLine("FontFamily must have 2 fonts with density 72 and 144.");
 
   do
   {
@@ -673,7 +678,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
       {
         /* skip blank lines and comment lines */
         ParseLine(s1, &token, &value);
-        if (token < g_glyph)
+        if (token <= h_leading)
         {
           if (curChar >= 0)
             ErrorX("Header must precede glyphs");
@@ -690,7 +695,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
         if (token == g_glyph)
         {
           if (curChar >= 0)
-            CloseGlyph(header, &width, &row, &col, autoWidth, autoRectWidth);
+            CloseGlyph(header, width, &row, &col, autoWidth, autoRectWidth);
 
           if (value == -1)
           {
@@ -735,7 +740,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
             ErrorX("Unexpected WIDTH token");
           fntOW[fntNo][curChar].width = value;
           if (singleOW && value * firstEntry->density != singleOW[curChar].width * fontFamilyEntries->density)
-            *badGlyph++ = curChar;
+            WarningLine("Widths of '%c' (%d) glyphs not in proportion across different densities", (isprint(curChar))? curChar : '?', curChar);
           continue;
         }
 
@@ -748,7 +753,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
             {
               fntOW[fntNo][curChar].width = width;
               if (singleOW && width * firstEntry->density != singleOW[curChar].width * fontFamilyEntries->density)
-                *badGlyph++ = curChar;
+                WarningLine("Widths of '%c' (%d) glyphs not in proportion across different densities", (isprint(curChar))? curChar : '?', curChar);
             }
           }
           else if (width != (int)strlen(s1))
@@ -773,7 +778,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
     }
 
     if (!feof(in))
-      Error2("Error reading file: ", filename);
+      Error("Error reading file: %s", filename);
 
     fclose(in);
 
@@ -801,7 +806,7 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
      */
 
     coltable[curChar + 1] = coltable[curChar] + fntOW[fntNo][curChar].width;
-    CloseGlyph(header, &width, &row, &col, autoWidth, autoRectWidth);
+    CloseGlyph(header, width, &row, &col, autoWidth, autoRectWidth);
     fntH[fntNo] = header[h_fRectHeight];
     if (singleOW &&
         fntH[fntNo]*firstEntry->density != singleH*fontFamilyEntries->density)
@@ -904,6 +909,8 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
       free(bitmap[x]);
     }
 
+    free(filename);
+
     // next entry
     fontFamilyEntries++;
   }
@@ -916,16 +923,6 @@ DumpFontFamily( int fntNo, int version, unsigned int densityCount, FNTFAMDEF * f
     EmitL(0xF2000000);
   }
 
-  if (badGlyph > badGlyphs)
-  {
-    char msg[2048];
-    int *ip;
-    strcpy(msg, "Glyph widths not in proportion across different densities:");
-    for (ip = badGlyphs; ip < badGlyph; ip++)
-      sprintf(strchr(msg, '\0'), (isprint(*ip))? " '%c'" : " %d", *ip);
-    WarningLine(msg);
-  }
-
   // Record the single density metrics for future reference
   free(fntOW[fntNo]);
   fntOW[fntNo] = singleOW;
