@@ -3,7 +3,7 @@
  * @(#)font.h
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2004, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -49,10 +49,10 @@ typedef struct FNTFAMDEF
 }
 FNTFAMDEF;
 
-extern int (*pfnChkCode) (unsigned char *cp,
+extern int (*pfnChkCode) (const unsigned char *cp,
                           int *pdx);
 
-extern void DumpFont(char *,
+extern void DumpFont(const char *,
                      int);
 
 extern void DumpFontFamily(int fntNo,
@@ -61,7 +61,7 @@ extern void DumpFontFamily(int fntNo,
                            FNTFAMDEF * fontFamilyEntries);
 
 extern void InitFontMem(int);
-extern void FreeFontMem();
+extern void FreeFontMem(void);
 
 extern int DxCalcRgdx(unsigned char *,
                       int,
@@ -71,6 +71,6 @@ extern int DxCalcExtent(unsigned char *,
 extern int DxChar(int,
                   int);
 extern int DyFont(int);
-extern int GetFontType();
+extern int GetFontType(void);
 
 #endif
