@@ -3,7 +3,7 @@
  * @(#)pilrc.h
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2004, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -119,7 +119,7 @@ EndEnum AlertType;
 
 Enum(FormObjectKind)
 {
-  frmFieldObj, frmControlObj, frmListObj, frmTableObj, frmBitmapObj, frmLineObj, frmFrameObj, frmRectangleObj, frmLabelObj, frmTitleObj, frmPopupObj, frmGraffitiStateObj, frmGadgetObj, frmScrollbarObj, frmSliderObj, /* RMa add support sliders */
+  frmFieldObj, frmControlObj, frmListObj, frmTableObj, frmBitmapObj, frmLineObj, frmFrameObj, frmRectangleObj, frmLabelObj, frmTitleObj, frmPopupObj, frmGraffitiStateObj, frmGadgetObj, frmScrollBarObj, frmSliderObj, /* RMa add support sliders */
     frmGraphicalControlObj                       /* RMa add support graphical control */
 }
 
@@ -1599,8 +1599,8 @@ Using these macros allows to avoid to test vfLE32 and write the code twice...
 -------------------------------------------------------------RNi-------------*/
 #define BAFIELD(obj, field) (vfLE32 ? obj.s32.field : obj.s16.field)
 #define PBAFIELD(pobj, field) (vfLE32 ? pobj->s32.field : pobj->s16.field)
-#define SETBAFIELD(obj, field, value) do { if (vfLE32) obj.s32.field = (value); else obj.s16.field = (value); } while (0)
-#define SETPBAFIELD(pobj, field, value) do { if (vfLE32) pobj->s32.field = (value); else pobj->s16.field = (value); } while (0)
+#define SETBAFIELD(obj, field, value)   /*lint -e{717}*/ do { if (vfLE32) obj.s32.field = (value); else obj.s16.field = (value); } while (0)
+#define SETPBAFIELD(pobj, field, value) /*lint -e{717}*/ do { if (vfLE32) pobj->s32.field = (value); else pobj->s16.field = (value); } while (0)
 #define BAFIELD16(obj, field) obj.s16.field
 #define BAFIELD32(obj, field) obj.s32.field
 #define PBAFIELD16(obj, field) obj->s16.field
@@ -1634,8 +1634,6 @@ FRMBA32Type;
      }
 FRM;
 
-     void ErrorLine(char *sz);
-
 /*
  * Translation Entry -- used to map to foreign languages 
  */
@@ -1680,12 +1678,12 @@ RCPFILE;
        rwBTN, rwPBN, rwCBX, rwPUT, rwSLT, rwREP, rwSLD, rwSCL,
 
        rwForm, rwBegin, rwEnd, rwModal, rwSaveBehind, rwNoSaveBehind,
-       rwHelpId, rwDefaultBtnId, rwMenuId,
+       rwHelpId, rwDefaultBtnId, rwMenuId, rwNoGSI,
        rwEnabled, rwDisabled, rwUsable, rwNonUsable, rwLeftAnchor,
        rwExtended, rwNonExtended,                /* MBr add: gadget */
        rwRightAnchor, rwGroup, rwFont,
        rwFrame, rwNoFrame, rwBoldFrame, rwRectFrame, rwVertical, rwGraphical,
-
+       
        rwEditable, rwNonEditable, rwUnderlined, rwSingleLine,
        rwMultipleLines, rwDynamicSize, rwLeftAlign, rwHasScrollBar,
        rwRightAlign, rwMaxChars,
@@ -1875,6 +1873,7 @@ RCPFILE;
        rwInclude,
        rwDefine,
        rwEqu,
+       rwUndef,
        rwIfdef,
        rwIfndef,
        rwElse,
@@ -1995,6 +1994,7 @@ RWT;
        {"helpid", NULL, rwHelpId},
        {"defaultbtnid", "defaultbutton", rwDefaultBtnId},       /* RMa form & alert */
        {"menuid", NULL, rwMenuId},
+       {"nograffitistateindicator", NULL, rwNoGSI}, /* No Graffiti State */
 
        {"enabled", NULL, rwEnabled},
        {"disabled", NULL, rwDisabled},
@@ -2216,6 +2216,7 @@ RWT;
        {"define", NULL, rwDefine},
        {"equ", NULL, rwEqu},
        {"include", NULL, rwInclude},
+       {"undef", NULL, rwUndef},
        {"ifdef", NULL, rwIfdef},
        {"ifndef", NULL, rwIfndef},
        {"else", NULL, rwElse},
@@ -2561,7 +2562,7 @@ ITM;
 #define if2AutoShift			0x00000080
 #define if2Scrollbar			0x00000100
 #define if2Numeric				0x00000200
-/* unused                       0x00000400 */
+#define if2GSI                  0x00000400 /* used with ifPt for GSI size calculation */
 #define if2Type 				0x00000800
 #define if2File 				0x00001000
 #define if2CreatorID			0x00002000
@@ -2662,11 +2663,22 @@ ITM;
 /*
  * Parse globals 
  */
+
+typedef struct INPUTCONTEXT
+{
+  char *szFilename;
+  FILE *fh;
+  int line;
+  char buffer[4096];
+  char *pch;
+  const char *pchLex;
+}
+INPUTCONTEXT;
+
 extern BOOL fTokUngotten;
 extern TOK tokPrev;
 extern TOK tok;
-extern int iline;
-extern char szLine[4096];
+extern INPUTCONTEXT vIn;
 
 extern BOOL vfWinGUI;
 extern BOOL vfQuiet;
@@ -2722,6 +2734,8 @@ extern BOOL vfStripNoLocRes;
 #define maxCategories  16
 #define categoryLength 16
 #define maxSafeResourceSize 65000
+#define	kGsiWidth      9
+#define kGsiHeight     10
 
 void ParseToFinalEnd(void);
 BOOL ObjectDesiredInOutputLocale(const ITM * itm);
@@ -2744,15 +2758,16 @@ int CbEmitStruct(void *pv,
                  char **ppchText,
                  BOOL fEmit);
 int CbStruct(char *szPic);
-RCPFILE *ParseFile(char *szIn,
-                   char *szOutDir,
-                   char *szResFile,
-                   char *szIncFile,
-                   int fontType);
+RCPFILE *ParseFile(const char *szIn,
+          const char *szOutDir,
+          const char *szResFile,
+          const char *szIncFile,
+          int fontType);
 SYM *PsymLookupId(int id);
-VOID AddSym(char *sz,
-            int wVal);
-VOID AddSymString(char* sz, char* val);
+VOID AddSym(const char *sz, int wVal);
+VOID AddSymString(const char* sz, const char* val);
+VOID RemoveSym(const char *sz);
+
 VOID AddDefineSymbol(void);
 
 #endif                                           // _pilrc_h
