@@ -3,7 +3,7 @@
  * @(#)plex.c
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2001, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -36,14 +36,14 @@
 #include <errno.h>
 #include "pilrc.h"
 
-void *
+static void *
 PlexGetElementAtUnsafe(PLEX * pplex,
                        int i)
 {
   return (void *)((unsigned char *)(pplex->m_rgb) + i * pplex->m_cbItem);
 }
 
-BOOL
+static BOOL
 PlexSetMax(PLEX * pplex,
            int iMax)
 {
