@@ -1,9 +1,8 @@
-
 /*
  * @(#)util.c
  *
  * Copyright 1997-1999, Wes Cherry   (mailto:wesc@technosis.com)
- *           2000-2003, Aaron Ardiri (mailto:aaron@ardiri.com)
+ *           2000-2004, Aaron Ardiri (mailto:aaron@ardiri.com)
  * All rights reserved.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -41,8 +40,6 @@
 #include <errno.h>
 #include <time.h>
 #include "pilrc.h"
-#include "util.h"
-#include "font.h"
 
 #ifdef WIN32
 #define DIRSEPARATOR  '\\'
@@ -53,11 +50,8 @@
 /*
  * Globals 
  */
-FILE *vfhIn;
-char szInFile[256];
 char rgbZero[16];
 BOOL vfErr;
-extern int iline;
 
 /*
  * Globals for output file handling
@@ -69,12 +63,12 @@ static int ibTotalOut;                           /* total output offset (after .
 
 static FILE *vfhRes = NULL;                      /* file receiving resource file ("-R") output */
 
-static char szOutFileDir[FILENAME_MAX];          /* directory for *.bin files */
+static const char *szOutFileDir = NULL;          /* directory for *.bin files */
 
-static char szOutResDBFile[FILENAME_MAX];        /* filename for final .ro file */
-static char szTempFile[FILENAME_MAX];            /* temporary filename */
+static char *szOutResDBFile = NULL;              /* filename for final .ro file */
+static char *szTempFile = NULL;                  /* temporary filename */
 
-static VOID WriteOutResourceDB();
+static VOID WriteOutResourceDB(void);
 
 DEFPL(PLEXRESOURCEDIR)
 typedef struct RESOURCEDIRENTRY
@@ -82,8 +76,7 @@ typedef struct RESOURCEDIRENTRY
   int type[4];
   int id;
   int offset;
-}
-RESOURCEDIRENTRY;
+} RESOURCEDIRENTRY;
 
 #define szRESOURCEDIRENTRY "b4,w,l"
 
@@ -127,125 +120,153 @@ void FreeAccessPathsList(void)
   allocatedIncludePaths = 0;
 }
 
+
+#ifndef CW_PLUGIN
 /*-----------------------------------------------------------------------------
-|	Error
-|	
-|		Report error, exit
+|	Diagnostic
+|
+|		Print error or warning, and perhaps exit. All error reporting
+|		is funneled via this function -- it is the only diagnostic
+|		function that things like the CW plugin need to override.
+-------------------------------------------------------------JohnM-----------*/
+VOID
+Diagnostic(BOOL fError, const char *filename, int lineno,
+           const char *szFormat, va_list *args)
+{
+  if (filename)
+    fprintf(stderr, vfVSErrors? "%s(%d): %s : " : "%s:%d: %s: ",
+            filename, lineno, fError? "error" : "warning");
+
+  vfprintf(stderr, szFormat, *args);
+  fprintf(stderr, "\n");
+
+  if (fError)
+    exit(1);
+}
+#endif
+
+/*-----------------------------------------------------------------------------
+|	Error, ErrorLine, WarningLine
+|
+|		Call Diagnostic() with appropriate arguments
 ------------------------------------------------------------WESC------------*/
 VOID
-Error(char *sz)
+Error(const char *szFormat, ...)
 {
-#ifdef WINGUI
-  if (vfWinGUI)
-  {
-    if (!vfErr)
+  va_list args;
+  va_start(args, szFormat);
+  Diagnostic(fTrue, NULL, 0, szFormat, &args);
+  va_end(args);
+}
+
+VOID
+ErrorLine(const char *szFormat, ...)
+{
+  va_list args;
+  va_start(args, szFormat);
+  Diagnostic(fTrue, vIn.szFilename, vIn.line, szFormat, &args);
+  va_end(args);
+}
+
+VOID
+WarningLine(const char *szFormat, ...)
+{
+  va_list args;
+  va_start(args, szFormat);
+  Diagnostic(fFalse, vIn.szFilename, vIn.line, szFormat, &args);
+  va_end(args);
+}
+
+
+/*-----------------------------------------------------------------------------
+|	MakeFilename
+|
+|		Construct a filename and return it in newly malloc()ed memory
+-------------------------------------------------------------JOHN------------*/
+char *
+MakeFilename(const char *szFormat, ...)
+{
+  const char *s;
+  char *fname, *fn;
+  va_list args;
+  size_t len = 0;
+
+  va_start(args, szFormat);
+
+  for (s = szFormat; *s; s++)
+    if (*s == '%')
     {
-      vfErr = fTrue;
-      MessageBox(NULL, sz, NULL, MB_OK);
+      char *arg = va_arg(args, char *);
+      if (*++s == 's')
+        len += strlen(arg);
     }
-  }
-  else
-#elif defined(CW_PLUGIN)
-  if (1)
-    CWError(sz);
-  else
-#endif
-  {
-    fprintf(stderr, "%s\n", sz);                   // added "%s", AA
-    exit(1);
-  }
+    else
+      len++;
+
+  va_end(args);
+
+  fname = malloc(len + 1);
+  if (fname == NULL)
+    Error("Out of memory");
+
+  va_start(args, szFormat);
+
+  fn = fname;
+  for (s = szFormat; *s; s++)
+    switch (*s)
+    {
+      case '%':
+        switch (*++s)
+        {
+          case 's':
+            strcpy(fn, va_arg(args, char *));
+            fn = strrchr(fn, '\0');
+            break;
+
+          case 'e':
+            {
+              char *ext = va_arg(args, char *);
+              size_t extlen = strlen(ext);
+              *fn = '\0';
+              if (strlen(fname) >= extlen && FSzEqI(fn - extlen, ext))
+                fn -= extlen;
+            }
+            break;
+        }
+        break;
+
+      case '/':
+        if (fn > fname && fn[-1] != '/' && fn[-1] != '\\')
+          *fn++ = DIRSEPARATOR;
+        break;
+
+      default:
+        *fn++ = *s;
+        break;
+    }
+
+  *fn = '\0';
+
+  va_end(args);
+  return fname;
 }
 
 /*-----------------------------------------------------------------------------
-|	Error2
--------------------------------------------------------------WESC------------*/
-VOID
-Error2(char *sz1,
-       char *sz2)
+|	MakeTempFilename
+|
+|		Return a temporary filename in newly malloc()ed memory
+-------------------------------------------------------------JOHN------------*/
+char *
+MakeTempFilename(void)
 {
-  char szT[256];
-
-  strcpy(szT, sz1);
-  strcat(szT, sz2);
-  Error(szT);
+  /* In some situations, the race condition inherent in the use of tmpnam()
+     is a security risk.  This is not a significant issue for PilRC.  */
+  char *fname = tmpnam(NULL);
+  if (fname == NULL)
+    Error("Can't make temporary filename -- tmpnam() failed");
+  return MakeFilename("%s", fname);
 }
 
-/*-----------------------------------------------------------------------------
-|	Error3
--------------------------------------------------------------WESC------------*/
-VOID
-Error3(char *sz1,
-       char *sz2,
-       char *sz3)
-{
-  char szT[256];
-
-  strcpy(szT, sz1);
-  strcat(szT, sz2);
-  strcat(szT, " ");
-  strcat(szT, sz3);
-  Error(szT);
-}
-
-/*-----------------------------------------------------------------------------
-|	ErrorLine2
-|	
-|		Report error w/ current line #
--------------------------------------------------------------WESC------------*/
-VOID
-ErrorLine2(char *sz,
-           char *sz2)
-{
-#ifdef CW_PLUGIN
-  CWErrorLine2(sz, sz2, iline);
-#else
-  char szErr[256];
-
-  if (sz2 == NULL)
-    sprintf(szErr,
-            ((vfVSErrors)
-             ? "%s(%d): error : %s"
-             : "%s:%d: error : %s"), szInFile, iline, sz);
-  else
-    sprintf(szErr,
-            ((vfVSErrors)
-             ? "%s(%d): error : %s %s"
-             : "%s:%d: error : %s %s"), szInFile, iline, sz, sz2);
-  Error(szErr);
-#endif
-}
-
-/*-----------------------------------------------------------------------------
-|	ErrorLine
-|	
-|		Report error w/ current line #
--------------------------------------------------------------WESC------------*/
-VOID
-ErrorLine(char *sz)
-{
-  ErrorLine2(sz, NULL);
-}
-
-/*-----------------------------------------------------------------------------
-|	WarningLine
-|	
-|		Report error w/ current line #
--------------------------------------------------------------WESC------------*/
-VOID
-WarningLine(char *sz)
-{
-#ifdef CW_PLUGIN
-  CWWarningLine(sz, iline);
-#else
-  char szErr[256];
-
-  sprintf(szErr,
-          ((vfVSErrors)
-           ? "%s(%d): warning : %s\n"
-           : "%s:%d: warning : %s\n"), szInFile, iline, sz);
-  fputs(szErr, stderr);
-#endif
-}
 
 /*-----------------------------------------------------------------------------
 |	EmitB
@@ -320,7 +341,7 @@ EmitL(unsigned long l)
 |		Return current output file file offset
 -------------------------------------------------------------WESC------------*/
 int
-IbOut()
+IbOut(void)
 {
   return ibOut;
 }
@@ -417,7 +438,7 @@ DumpBytes(VOID * pv,
 |		Pads output to a word or a long boundary by emitting a 0 if necessary
 -------------------------------------------------------------WESC------------*/
 VOID
-PadBoundary()
+PadBoundary(void)
 {
   if (vfLE32)
   {
@@ -434,7 +455,7 @@ PadBoundary()
 |		Pads output to a word boundary by emitting a 0 if necessary
 -------------------------------------------------------------WESC------------*/
 VOID
-PadWordBoundary()
+PadWordBoundary(void)
 {
   if (ibOut & 1)
     DumpBytes(rgbZero, 1);
@@ -446,12 +467,12 @@ PadWordBoundary()
 |		Set output file path -- no trailing / or \ 
 -------------------------------------------------------------WESC------------*/
 VOID
-SetOutFileDir(char *sz)
+SetOutFileDir(const char *sz)
 {
-  if (sz == NULL || strcmp(sz, ".") == 0)
-    strcpy(szOutFileDir, "");
+  if (sz && strcmp(sz, ".") == 0)
+    szOutFileDir = "";
   else
-    sprintf(szOutFileDir, "%s%c", sz, DIRSEPARATOR);
+    szOutFileDir = sz;
 }
 
 /*-----------------------------------------------------------------------------
@@ -463,24 +484,21 @@ SetOutFileDir(char *sz)
 static VOID
 RemoveTempFile(VOID)
 {
-  if (*szTempFile)
+  if (szTempFile)
   {
     remove(szTempFile);
-    strcpy(szTempFile, "");
+    free(szTempFile);
+    szTempFile = NULL;
   }
 }
 
 VOID
-OpenResDBFile(char *sz)
+OpenResDBFile(const char *sz)
 {
   static BOOL registered = fFalse;
 
-  strcpy(szOutResDBFile, sz);
-
-  strcpy(szTempFile, szOutResDBFile);
-  szTempFile[strlen(szTempFile) - 1] = '~';
-
-  remove(szTempFile);
+  szOutResDBFile = MakeFilename("%s", sz);
+  szTempFile = MakeTempFilename();
 
   if (!registered)
   {
@@ -492,7 +510,7 @@ OpenResDBFile(char *sz)
 }
 
 VOID
-CloseResDBFile()
+CloseResDBFile(void)
 {
   if (!vfInhibitOutput)
   {
@@ -502,13 +520,14 @@ CloseResDBFile()
       Assert(vfhOut == NULL);
       vfhOut = fopen(szOutResDBFile, "wb");
       if (vfhOut == NULL)
-        Error3("Unable to open output resource DB:", szOutResDBFile, strerror(errno));
+        Error("Unable to open output resource DB %s: %s", szOutResDBFile, strerror(errno));
 
       WriteOutResourceDB();
       fclose(vfhOut);
   }
   
   RemoveTempFile();
+  free(szOutResDBFile);
   PlexFree(&resdir);
 }
 
@@ -531,23 +550,20 @@ OpenOutput(char *szBase,
   CWOpenOutput(szBase, id);
   ibOut = 0;
 #else
-  char szPrettyName[FILENAME_MAX];
-  char *szFileName;
-  char *szMode;
+  char szBinFileName[4 + 4 + 4 + 1];
 
   // save for possible errors
   memset(outputResStr, 0, 5);
   strncpy(outputResStr, szBase, 4);
   outputResID = id;
 
-  /*
-   * #ifdef BINOUT 
-   */
   if (vfInhibitOutput || vfWinGUI)
     return;
 
   Assert(vfhOut == NULL);
 
+  sprintf(szBinFileName, "%.4s%04x.bin", szBase, id);
+
   if (vfPrc)
   {
     RESOURCEDIRENTRY entry;
@@ -557,33 +573,38 @@ OpenOutput(char *szBase,
     entry.offset = ibTotalOut;
     PlexAddElement(&resdir, &entry);
 
-    sprintf(szPrettyName, "temporary %s%04x.bin", szBase, id);
-    szFileName = szTempFile;
-    szMode = "ab";
+    vfhOut = fopen(szTempFile, "ab");
+    if (vfhOut == NULL)
+      Error("Unable to open binary file %s: %s", szTempFile, strerror(errno));
+
+    if (!vfQuiet)
+      printf("Writing temporary %s ", szBinFileName);
+
+    if (vfhRes)
+      fprintf(vfhRes, "\tres '%s', %d, \"%s(%s)\"\n", szBase, id,
+              szOutResDBFile, szBinFileName);
   }
   else
   {
-    sprintf(szPrettyName, "%s%s%04x.bin", szOutFileDir, szBase, id);
-    szFileName = szPrettyName;
-    szMode = "w+b";
+    char *szBinPath;
+    if (szOutFileDir == NULL)
+      Error("Can't happen (OutFileDir is not set)");
+    szBinPath = MakeFilename("%s/%s", szOutFileDir, szBinFileName);
+
+    vfhOut = fopen(szBinPath, "w+b");
+    if (vfhOut == NULL)
+      Error("Unable to open binary file %s: %s", szBinPath, strerror(errno));
+
+    if (!vfQuiet)
+      printf("Writing %s ", szBinPath);
+
+    if (vfhRes)
+      fprintf(vfhRes, "\tres '%s', %d, \"%s\"\n", szBase, id, szBinPath);
+
+    free(szBinPath);
   }
 
-  vfhOut = fopen(szFileName, szMode);
-  if (vfhOut == NULL)
-    Error3("Unable to open binary file:", szFileName, strerror(errno));
-
-  if (!vfQuiet)
-    printf("Writing %s ", szPrettyName);
   ibOut = 0;
-
-  /*
-   * #endif 
-   */
-
-  if (vfhRes)
-  {
-    fprintf(vfhRes, "\tres '%s', %d, \"%s\"\n", szBase, id, szPrettyName);
-  }
 #endif
 }
 
@@ -591,7 +612,7 @@ OpenOutput(char *szBase,
 |	CloseOutput
 -------------------------------------------------------------WESC------------*/
 VOID
-CloseOutput()
+CloseOutput(void)
 {
 #ifdef CW_PLUGIN
   CWCloseOutput();
@@ -612,14 +633,9 @@ CloseOutput()
     printf("(%d bytes)\n", ibOut);
   
   if (!vfAllowLargeResources && ibOut > maxSafeResourceSize)
-  {
-    char buffer[120];
-  	sprintf(buffer,
-  		"Resource '%s' %d, %d bytes, exceeds safe "
+    WarningLine("Resource '%s' %d, %d bytes, exceeds safe "
   		"HotSync size limit of %d bytes",
   		outputResStr, outputResID, ibOut, maxSafeResourceSize);
-  	WarningLine(buffer);
-  }
 
   ibTotalOut += ibOut;
   if (vfhOut != NULL)
@@ -638,7 +654,7 @@ CloseOutput()
 |	getOpenedOutputFile
 -------------------------------------------------------------RMa-------------*/
 FILE *
-getOpenedOutputFile()
+getOpenedOutputFile(void)
 {
   if (vfhOut == NULL)
     Error("No output file opened");
@@ -646,7 +662,7 @@ getOpenedOutputFile()
 }
 
 VOID
-OpenResFile(char *sz)
+OpenResFile(const char *sz)
 {
   if (vfInhibitOutput || vfWinGUI)
     return;
@@ -657,13 +673,13 @@ OpenResFile(char *sz)
   vfhRes = fopen(sz, "wt");
 
   if (vfhRes == NULL)
-    Error3("Unable to open res file:", sz, strerror(errno));
+    Error("Unable to open res file %s: %s", sz, strerror(errno));
   if (!vfQuiet)
     printf("Generating res file: %s\n", sz);
 }
 
 VOID
-CloseResFile()
+CloseResFile(void)
 {
   if (vfInhibitOutput || vfWinGUI)
     return;
@@ -679,51 +695,39 @@ CloseResFile()
 |	FindAndOpenFile
 -------------------------------------------------------------DAVE------------*/
 char *
-FindAndOpenFile(char *szIn,
-                char *mode,
+FindAndOpenFile(const char *szIn,
+                const char *mode,
                 FILE ** returnFile)
 {
-  FILE *file = fopen(szIn, mode);
+  char *szFullName = NULL;
 
-  if (file == NULL)
+  if ((*returnFile = fopen(szIn, mode)) != NULL)
   {
-    static char szFullName[FILENAME_MAX];
-    int i;
-
-    for (i = 0; i < totalIncludePaths; i++)
-    {
-      sprintf(szFullName, "%s%c%s", includePaths[i], DIRSEPARATOR, szIn);
-
-      file = fopen(szFullName, mode);
-      if (file != NULL)
-      {
-        break;
-      }
-    }
-
-    if (i == totalIncludePaths)
-    {
-      ErrorLine2("Unable to find ", szIn);
-    }
-
-    if (vfTrackDepends)
-    {
-        AddEntryToDependsList(szFullName);
-    }
-
-    *returnFile = file;
-    return szFullName;
+    szFullName = MakeFilename("%s", szIn);
   }
   else
   {
-    if (vfTrackDepends)
+    int i;
+    for (i = 0; i < totalIncludePaths; i++)
     {
-        AddEntryToDependsList(szIn);
-    }
+      szFullName = MakeFilename("%s/%s", includePaths[i], szIn);
+      if ((*returnFile = fopen(szFullName, mode)) != NULL)
+        break;
 
-    *returnFile = file;
-    return szIn;
+      free(szFullName);
+    }
   }
+
+  if (*returnFile == NULL)
+  {
+    ErrorLine("Unable to find %s", szIn);
+    return NULL;
+  }
+
+  if (vfTrackDepends)
+    AddEntryToDependsList(szFullName);
+
+  return szFullName;
 }
 
 /*
@@ -799,13 +803,12 @@ typedef struct DBHEADER
   //p_int uidseed;     /* zl */
   //p_int nextlist;    /* zl */
   p_int nrecords;                                /* w */
-}
-DBHEADER;
+} DBHEADER;
 
 #define szDBHEADER "b32,w,w,l,l,zl4,b4,b4,zl2,w"
 
 static VOID
-WriteOutResourceDB()
+WriteOutResourceDB(void)
 {
   DBHEADER head;
   char buf[4096];
@@ -876,7 +879,7 @@ WriteOutResourceDB()
 
   f = fopen(szTempFile, "rb");
   if (f == NULL)
-    Error3("Unable to open resource DB:", szTempFile, strerror(errno));
+    Error("Unable to open resource DB %s: %s", szTempFile, strerror(errno));
 
   while ((n = fread(buf, 1, sizeof buf, f)) > 0)
     DumpBytes(buf, n);
@@ -894,17 +897,6 @@ struct DependsNode
 
 static struct DependsNode *pdDependsRoot = NULL;
 
-static char szDependsTarget[FILENAME_MAX] = "";
-
-/*-----------------------------------------------------------------------------
-|      SetDependsTarget
--------------------------------------------------------------BLC-------------*/
-
-void SetDependsTarget(const char *szIn)
-{
-    strcpy(szDependsTarget, szIn);
-}
-
 /*-----------------------------------------------------------------------------
 |      InitDependsList
 -------------------------------------------------------------BLC-------------*/
@@ -976,7 +968,7 @@ static char *EscapeChars(char *dest, const char *src)
 |      OutputDependsList
 -------------------------------------------------------------BLC-------------*/
 
-void OutputDependsList(FILE *dependsFile)
+void OutputDependsList(FILE *dependsFile, const char *szDependsTarget)
 {
     char buffer[512];
     struct DependsNode *pdNode;
